@@ -244,17 +244,19 @@ ssize_t ps_read( options_t *options, ps_t *ps, void *ts )
                 {
                     /* MPEG-2 */
                     ps->scr = ((u64)(ps->ps_data[4] & 0x38) << 27) |
-                         ((u64)(U32_AT(ps->ps_data + 4) & 0x03FFF800)
-                                        << 4) |
-                         ((( ((u32)U16_AT(ps->ps_data + 6) << 16)
-                            | (u32)U16_AT(ps->ps_data + 8) ) & 0x03FFF800)
-                                        >> 11);
+                              ((u64)(ps->ps_data[4] & 0x3) << 28) |
+                              ((u64)(ps->ps_data[5] & 0xff) << 20) |
+                              ((u64)(ps->ps_data[6] & 0xf8) << 12) |
+                              ((u64)(ps->ps_data[6] & 0x3) << 13) |
+                              ((u64)(ps->ps_data[7] & 0xff) << 5) |
+                              ((u64)(ps->ps_data[8] & 0xf8) >> 3);
                 }
                 else
                 {
                     /* MPEG-1 */
                     ps->scr = ((u64)(ps->ps_data[4] & 0x0E) << 29) |
-                         (((u64)U32_AT(ps->ps_data + 4) & 0xFFFE00) << 6) |
+                         ((u64)ps->ps_data[5] << 22) |
+                         (((u64)ps->ps_data[6] & 0xFE) << 14) |
                          ((u64)ps->ps_data[7] << 7) |
                          ((u64)ps->ps_data[8] >> 1);
                 }
