@@ -4,11 +4,13 @@
  * Copyright (C) 1998, 1999, 2000 VideoLAN
  *
  * Authors:
- * Regis Duchesne,     VIA, ECP, France <regis@via.ecp.fr>,  27/01/97
- * Michel Lespinasse,  VIA, ECP, France <walken@via.ecp.fr>, 02/02/97
- * Jean-Marc Dressler, VIA, ECP, France <polux@via.ecp.fr>,  17/05/99
- * Samuel Hocevar,     VIA, ECP, France <sam@via.ecp.fr>,    17/12/99
- * Damien Lucas,       VIA, ECP, France <nitrox@via.ecp.fr>, 30/05/01
+ * Regis Duchesne,     VIA, ECP, France <regis@via.ecp.fr>,   27/01/97
+ * Michel Lespinasse,  VIA, ECP, France <walken@via.ecp.fr>,  02/02/97
+ * Jean-Marc Dressler, VIA, ECP, France <polux@via.ecp.fr>,   17/05/99
+ * Samuel Hocevar,     VIA, ECP, France <sam@via.ecp.fr>,     17/12/99
+ * Damien Lucas,       VIA, ECP, France <nitrox@via.ecp.fr>,  30/05/01
+ * Marc Ariberti,      VIA, ECP, France <marcari@via.ecp.fr>, 29/04/02
+ * Christophe Massiot, VIA, ECP, France <massiot@via.ecp.fr>, 30/05/02
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,20 +43,20 @@
  *   delays our read from the disk.
  */ 
  
+/* common stuff */
+#include "vlms.h"
+
 #ifdef LINUX
 #  include <net/if.h>
 #endif
 
-/* common stuff */
-#include "vlms.h"
-
 /* TS parsing functions */
 #include "ts.h"
 /* PS/PES parsing functions */
 #include "ps.h"
 
 #define COPYRIGHT \
-"VideoLAN Mini Server - version 0.2.0 Onatopp - (c)1996-2000 VideoLAN\n"
+"VideoLAN Mini Server - version 0.2.2 Onatopp - (c)1996-2000 VideoLAN\n"
 
 #define VERSION \
 COPYRIGHT \
@@ -70,6 +72,7 @@ COPYRIGHT \
 "\n" \
 "Options:\n" \
 "  -d host[:port]           target address\n" \
+"  -t ttl                   ttl value\n" \
 "  -p pid                   optional PCR PID\n" \
 "\n" \
 "  -a [ac3|lpcm|mpeg|off]   choose audio type\n" \
@@ -77,6 +80,9 @@ COPYRIGHT \
 "  -c [0-15]                choose audio channel\n" \
 "  -s [0-31]                choose subtitle channel\n" \
 "\n" \
+"  -l                       loops for ever\n" \
+"  -n nbloops               loops the playlist nbloops times\n" \
+"\n" \
 "  -h, --help               display this help and exit\n" \
 "  -v, --version            output version information and exit\n" \
 "\n" \
@@ -92,7 +98,7 @@ static const struct option longopts[] =
 };
 
 /* Short options */
-static const char *shortopts = "d:p:a:c:s:i:hv";
+static const char *shortopts = "d:p:t:a:c:s:i:n:lhv";
 
 /******************************************************************************
  * GetTime : returns the current time in microseconds
@@ -110,9 +116,14 @@ s64 GetTime(void)
  * ConvertPCRTime : extracts and converts the PCR time in microseconds
  ******************************************************************************/
 
-s64 ConvertPCRTime(file_ts_packet *pcr_buff)
+s64 ConvertPCRTime(file_ts_packet *_pcr_buff)
 {
-    return( (((((s64)U32_AT(((u8*)pcr_buff)+6)) << 1) | (((u8*)pcr_buff)[10] >> 7)) * 300) / 27 );
+    u8 * pcr_buff = (u8 *)_pcr_buff;
+    return((((s64)(pcr_buff[6]) << 25) |
+            ((s64)(pcr_buff[7]) << 17) |
+            ((s64)(pcr_buff[8]) << 9) |
+            ((s64)(pcr_buff[9]) << 1) |
+            ((s64)(pcr_buff[10]) >> 7)) * 300 / 27 );
 }
 
 /******************************************************************************
@@ -407,6 +418,7 @@ ssize_t safe_read( options_t *options, unsigned char *buf, int count )
             return ret;
 
         if( ret == 0 )
+        {
             /* zero means end of file */
             if( options->i_list_index )
             {
@@ -414,8 +426,23 @@ ssize_t safe_read( options_t *options, unsigned char *buf, int count )
             }
             else
             {
-                break;
+                /* test if we reached the number of loops */
+                /* a negative value means looping infinitely */
+                if( options->loop > 0 )
+                {
+                    options->loop--;
+                }
+                if( options->loop == 0)
+                {
+                    break;
+                }
+                else 
+                {
+                    options->i_list_index = options->i_list_size;
+                    file_next( options );
+                }
             }
+        }
 
         cnt += ret;
     }
@@ -444,6 +471,7 @@ int open_socket( struct s_options *options,
     {
         BuildInetAddr( remote_addr, options->host );
     }
+#ifdef LINUX
     else
     {
         int i_if_nb;
@@ -463,10 +491,32 @@ int open_socket( struct s_options *options,
         *remote_addr = net_descr.a_if[i_if_nb-1].sa_bcast_addr;
         free( net_descr.a_if );
     }
+#endif
 
+    if( options->ttl != 0 ) 
+    {
+        sockopt = options->ttl;
+        if( IN_CLASSD( ntohl( remote_addr->sin_addr.s_addr ) ) )
+        {
+            if( setsockopt( socket_fd, IPPROTO_IP, IP_MULTICAST_TTL, 
+                            (void *)&sockopt, sizeof(sockopt) ) < 0 )
+            {
+                err_sys( "IP_MULTICAST_TTL setsockopt TTL error" );
+            }
+        }
+        else 
+        {
+            if( setsockopt( socket_fd, IPPROTO_IP, IP_TTL, 
+                            (void *)&sockopt, sizeof(sockopt) ) < 0 )
+            {
+                err_sys( "IP_TTL setsockopt TTL error" );
+            }
+        }
+    }
+    
     sockopt = 1;
     if( setsockopt( socket_fd, SOL_SOCKET, SO_BROADCAST,
-                    &sockopt, sizeof(sockopt) ) < 0 )
+                    (void *)&sockopt, sizeof(sockopt) ) < 0 )
     {
         err_sys( "SO_BROADCAST setsockopt error" );
     }
@@ -482,7 +532,7 @@ int open_socket( struct s_options *options,
     }
 #endif
     
-    if( bind(socket_fd, &local_addr, sizeof(local_addr)) < 0 )
+    if( bind(socket_fd, (struct sockaddr *)&local_addr, sizeof(local_addr)) < 0 )
     {
         err_sys( "bind() error" );
     }
@@ -519,6 +569,8 @@ int main(int argc, char **argv)
     int opt;
 
     options.pcr_pid = 0;
+    options.ttl = 0;
+    options.loop = 1;
     options.host = NULL;
     options.dev = NULL;
     options.audio_type = REQUESTED_AC3;
@@ -540,6 +592,15 @@ int main(int argc, char **argv)
         case 'p':
             options.pcr_pid = atoi(optarg);
             break;
+        case 't':
+            options.ttl = atoi(optarg);
+            break;
+        case 'l':
+            options.loop = -1;
+            break;
+        case 'n':
+            options.loop = atoi(optarg);
+            break;
         case 'a' :
             if ( ! strcmp(optarg, "mpeg") )
                 options.audio_type = REQUESTED_MPEG;
@@ -604,6 +665,9 @@ int main(int argc, char **argv)
         options.i_list_index = 1;
     }
 
+    /* takes note of the length of the playlist for looping */
+    options.i_list_size = options.i_list_index;
+
     /* Don't use output buffer to have a real-time display */
     setvbuf( stdout, NULL, _IONBF ,0 );
 
